==> heptapod-rails-0.25.2.tar.gz: heptapod-rails-0.25.2/lib/gitlab/git/tree.rb <==
# frozen_string_literal: true

module Gitlab
  module Git
    class Tree
      include Gitlab::EncodingHelper
      extend Gitlab::Git::WrapsGitalyErrors

      attr_accessor :id, :root_id, :type, :mode, :commit_id, :submodule_url
      attr_writer :name, :path, :flat_path

      class << self
        # Get list of tree objects
        # for repository based on commit sha and path
        # Uses rugged for raw objects
        #
        # Gitaly migration: https://gitlab.com/gitlab-org/gitaly/issues/320
        def where(repository, sha, path = nil, recursive = false, pagination_params = nil)
          path = nil if path == '' || path == '/'

          tree_entries(repository, sha, path, recursive, pagination_params)
        end

        def tree_entries(repository, sha, path, recursive, pagination_params = nil)
          rpc_args = [repository, sha, path, recursive, pagination_params]
          wrapped_gitaly_errors do
            if repository.hgitaly1?
              git_sha = repository.gitsha_from_sha(sha)
              # sha can also be a ref, so let's keep using it if it's not
              # known as an actual Mercurial SHA
              rpc_args[1] = git_sha unless git_sha.nil?

              entries, cursor = repository.gitaly_commit_client.tree_entries(*rpc_args)
              entries.each do |tree|
                tree.hpd_back_convert!(repository)
              end
              [entries, cursor]
            else
              repository.gitaly_commit_client.tree_entries(*rpc_args)
            end
          end
        end

        private

        # Recursive search of tree id for path
        #
        # Ex.
        #   blog/            # oid: 1a
        #     app/           # oid: 2a
        #       models/      # oid: 3a
        #       views/       # oid: 4a
        #
        #
        # Tree.find_id_by_path(repo, '1a', 'app/models') # => '3a'
        #
        def find_id_by_path(repository, root_id, path)
          root_tree = repository.lookup(root_id)
          path_arr = path.split('/')

          entry = root_tree.find do |entry|
            entry[:name] == path_arr[0] && entry[:type] == :tree
          end

          return unless entry

          if path_arr.size > 1
            path_arr.shift
            find_id_by_path(repository, entry[:oid], path_arr.join('/'))
          else
            entry[:oid]
          end
        end
      end

      def initialize(options)
        %w(id root_id name path flat_path type mode commit_id).each do |key|
          self.send("#{key}=", options[key.to_sym]) # rubocop:disable GitlabSecurity/PublicSend
        end
      end

      # Transitional Hgitaly1 back conversion for given repository
      #
      # Heptapod specific, and transitional: will be useful only until
      # all native repositories are fully native.
      def hpd_back_convert!(repository)
        converted = repository.sha_from_gitsha(@commit_id)
        # commit_id can actually be a ref. In that case it will fail
        # lookup and we keep it as is (if it coincides with a valid full SHA,
        # it will be masked)
        @commit_id = converted unless converted.nil?
        self
      end

      def name
        encode! @name
      end

      def path
        encode! @path
      end

      def flat_path
        encode! @flat_path
      end

      def dir?
        type == :tree
      end

      def file?
        type == :blob
      end

      def submodule?
        type == :commit
      end

      def readme?
        name =~ /^readme/i
      end

      def contributing?
        name =~ /^contributing/i
      end
    end
  end
end

Gitlab::Git::Tree.singleton_class.prepend Gitlab::Git::RuggedImpl::Tree::ClassMethods

==> heptapod-rails-0.36.0rc1.tar.gz: heptapod-rails-0.36.0rc1/lib/gitlab/git/tree.rb <==
# frozen_string_literal: true

module Gitlab
  module Git
    class Tree
      include Gitlab::EncodingHelper
      extend Gitlab::Git::WrapsGitalyErrors

      attr_accessor :id, :root_id, :type, :mode, :commit_id, :submodule_url
      attr_writer :name, :path, :flat_path

      class << self
        # Get list of tree objects
        # for repository based on commit sha and path
        # Uses rugged for raw objects
        #
        # Gitaly migration: https://gitlab.com/gitlab-org/gitaly/issues/320
        def where(repository, sha, path = nil, recursive = false, skip_flat_paths = true, pagination_params = nil)
          path = nil if path == '' || path == '/'

          tree_entries(repository, sha, path, recursive, skip_flat_paths, pagination_params)
        end

        def tree_entries(repository, sha, path, recursive, skip_flat_paths, pagination_params = nil)
          rpc_args = [repository, sha, path, recursive, skip_flat_paths, pagination_params]
          wrapped_gitaly_errors do
            if repository.hgitaly1?
              git_sha = repository.gitsha_from_sha(sha)
              # sha can also be a ref, so let's keep using it if it's not
              # known as an actual Mercurial SHA
              rpc_args[1] = git_sha unless git_sha.nil?

              entries, cursor = repository.gitaly_commit_client.tree_entries(*rpc_args)
              entries.each do |tree|
                tree.hpd_back_convert!(repository)
              end
              [entries, cursor]
            else
              repository.gitaly_commit_client.tree_entries(*rpc_args)
            end
          end
        end

        private

        # Recursive search of tree id for path
        #
        # Ex.
        #   blog/            # oid: 1a
        #     app/           # oid: 2a
        #       models/      # oid: 3a
        #       views/       # oid: 4a
        #
        #
        # Tree.find_id_by_path(repo, '1a', 'app/models') # => '3a'
        #
        def find_id_by_path(repository, root_id, path)
          root_tree = repository.lookup(root_id)
          path_arr = path.split('/')

          entry = root_tree.find do |entry|
            entry[:name] == path_arr[0] && entry[:type] == :tree
          end

          return unless entry

          if path_arr.size > 1
            path_arr.shift
            find_id_by_path(repository, entry[:oid], path_arr.join('/'))
          else
            entry[:oid]
          end
        end
      end

      def initialize(options)
        %w(id root_id name path flat_path type mode commit_id).each do |key|
          self.send("#{key}=", options[key.to_sym]) # rubocop:disable GitlabSecurity/PublicSend
        end
      end

      # Transitional Hgitaly1 back conversion for given repository
      #
      # Heptapod specific, and transitional: will be useful only until
      # all native repositories are fully native.
      def hpd_back_convert!(repository)
        converted = repository.sha_from_gitsha(@commit_id)
        # commit_id can actually be a ref. In that case it will fail
        # lookup and we keep it as is (if it coincides with a valid full SHA,
        # it will be masked)
        @commit_id = converted unless converted.nil?
        self
      end

      def name
        encode! @name
      end

      def path
        encode! @path
      end

      def flat_path
        encode! @flat_path
      end

      def dir?
        type == :tree
      end

      def file?
        type == :blob
      end

      def submodule?
        type == :commit
      end

      def readme?
        name =~ /^readme/i
      end

      def contributing?
        name =~ /^contributing/i
      end
    end
  end
end

Gitlab::Git::Tree.singleton_class.prepend Gitlab::Git::RuggedImpl::Tree::ClassMethods
